
# q-expansion of newform 160.2.n.d, downloaded from the LMFDB on 11 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 160
weight = 2
poly_data = [1, 0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = [[31, [-1, 0]], [101, [1, 0]], [97, [0, 1]]]
aps_data = [[0, 0], [1, 1], [1, 2], [-3, 3], [0, -2], [3, -3], [1, 1], [4, 0], [-1, -1], [0, 0], [0, -10], [-1, -1], [-10, 0], [5, 5], [-3, 3], [-5, 5], [-12, 0], [2, 0], [-1, 1], [0, -2], [1, -1], [8, 0], [5, 5], [0, 16], [-3, -3], [6, 0], [-9, -9], [3, -3], [0, -4], [-3, 3], [-7, 7], [0, -10], [-11, -11], [12, 0], [0, -4], [0, 6], [-1, -1], [1, 1], [1, -1], [-5, 5], [-12, 0], [22, 0], [0, 14], [-15, 15], [-13, -13], [-16, 0], [0, 14], [-1, -1], [-5, 5], [0, -8], [21, -21], [16, 0], [-2, 0], [0, 6], [5, 5], [11, 11], [0, 20], [0, 14], [11, 11], [6, 0], [-7, -7], [11, -11], [-17, 17], [0, -18], [9, -9], [-13, -13], [0, -26], [-15, -15], [-9, 9], [0, -8], [-15, 15], [32, 0], [-15, 15], [-9, 9], [20, 0], [-1, -1], [0, -4], [15, 15], [-22, 0], [0, 20], [28, 0], [34, 0], [0, 30], [21, -21], [16, 0], [25, 25], [0, -12], [9, 9], [-2, 0], [11, 11], [-13, 13], [-40, 0], [-19, 19], [0, -10], [28, 0], [-17, -17], [0, 24], [-14, 0], [-15, -15], [30, 0], [11, -11], [27, 27], [33, 33], [0, 12], [0, -34], [-19, -19], [23, -23], [-7, 7], [0, 0], [-10, 0], [5, -5], [15, -15], [13, 13], [-12, 0], [0, 14], [30, 0], [-27, -27], [29, -29], [-1, 1], [-36, 0], [-30, 0], [-3, 3], [3, 3], [-11, -11], [0, 14], [34, 0], [0, 48], [16, 0], [-3, 3], [27, -27], [-28, 0], [-21, -21], [0, -2], [19, 19], [18, 0], [0, -8], [-17, 17], [31, -31], [-37, -37], [0, 16], [0, 54], [10, 0], [7, 7], [-25, 25], [0, 20], [-24, 0], [-9, 9], [-35, -35], [-28, 0], [-13, -13], [7, 7], [-18, 0], [33, 33], [5, -5], [27, -27], [0, 6], [-48, 0], [0, 28], [21, 21], [-34, 0], [31, -31], [-31, 31], [1, -1], [0, -2], [21, 21], [-5, -5], [0, -10], [-17, -17]]
